// meta data operations are non-void functions

typedef enum { False, True } Bool;

// design rule checking flags

#define stability   01         // elements have fixed storage locations
#define retrieval   02         // container is traversable
#define dlist_seen  04         // dlist component used in type equation
#define df_seen     010        // delflag component used in type equation

#define failure(pre,post) (pre & (~post)) 

// forward references

class CURSOR;
class CONTAINER;

// vertical links connecting cursor instances to container and
// component instances are done on a per-component basis.
// Presumably, the migration of vertical links in the chained
// structure could be moved up to this level (as pretty much
// all layers do the same thing


// in general, void-returning functions generate code.  
// int-returning functions are metadata (e.g., drc) functions
// and cursor/container/layer returning functions are used
// for type equation initialization.

class DS_container
{
   public:

   CONTAINER  *head;                     // head of list

   virtual void  elemxform  (void)  = 0;
   virtual void  cursxform  (void)  = 0;
   virtual void  contxform  (void)  = 0;
   virtual void  contdecl   (void)  = 0;
   virtual void  open_cont  (void)  = 0;
   virtual void  close_cont (void)  = 0;
};

class DS_cursor
{
   public:

   CURSOR          *head;                 // head of list
   DS_container    *cont;                 // pointer to container
   Bool            scan;                  // is cursor scan or not?


   virtual void cursdecl  (void)           = 0;
   virtual void init_curs (void)           = 0;
   virtual void insert    (char* buffer)   = 0;
   virtual void foreach   (void)           = 0;
   virtual void remove    (void)           = 0;
   virtual void ref       (char* field)    = 0;
   virtual void getrec    (char* buffer)   = 0;
};

realm DS
{
   public:

   virtual int           drc (int) = 0;
   virtual DS_container *make_container (CONTAINER* head) = 0;
   virtual DS_cursor    *make_cursor
                            (DS_container* cont, Bool scan, CURSOR* head) 
                             = 0;
};

