#include "headers.hh"
#include "components.hh"

extern "C" {
#include <unistd.h>
#include <stdlib.h>
}

#define CR     cout << ";\n"
#define Cr     cout << "\n"

int main( int argc, char *argv[] )
{
   int  i;
   char *last;
   TEQN teqn;
   DS_cursor    *all, *old;
   DS_container *emp_cont;
   CURSOR     *Allcursor, *Oldcursor;
   CONTAINER  *Emp_cont;

   if (argc == 1) {
      cout << "Usage: " << argv[0] << " <sequence of layers>\n";
      cout << "       DS realm = { text, array, malloc, delflag[DS], dlist[DS] }\n";
      exit(1);
   }

   // link components together in type equation

   while (argc-- > 1) {
      last = argv[ argc ];
      if (strcmp("text",last) == 0)
      {  teqn.lower = new text(teqn.lower); continue; };
      if (strcmp("array",last) == 0)
      {  teqn.lower = new array(teqn.lower); continue; };
      if (strcmp("malloc",last) == 0)
      {  teqn.lower = new umalloc(teqn.lower); continue; };
      if (strcmp("delflag",last) == 0)
      {  teqn.lower = new delflag(teqn.lower); continue; };
      if (strcmp("dlist",last) == 0)
      {  teqn.lower = new dlist(teqn.lower); continue; };
      cout << "unrecognized layer: " << last << "\n";
      exit( 1 );
   }

   // now design rule check equation.

   teqn.drc();

   // now initialize the te, container, and
   // cursor lists, one per each cursor, container variable

   Emp_cont  = teqn.make_container("emp_cont");
   Allcursor = teqn.make_cursor( Emp_cont, True, "all_emp" );
   Oldcursor = teqn.make_cursor( Emp_cont, False, "old_emp" );

   // now generate the marty program

   cout << "typedef struct {\n" ;
   cout << "   int   empno;\n" ;
   cout << "   int   age;\n" ;
   cout << "   char  dept_name[20];\n" ;
   cout << "   char  name[22];\n" ;
   cout << "} EMP_TYPE;\n" ;
   cout << "\n" ;
   cout << "typedef struct ele {\n" ;
   cout << "   int   empno;\n" ;
   cout << "   int   age;\n" ;
   cout << "   char  dept_name[20];\n" ;
   cout << "   char  name[22];\n" ;
   cout << "   "; Emp_cont->elemxform(); CR;
   cout << "} ELE_TYPE;\n" ;
   cout << "\n" ;
   cout << "typedef struct {\n" ;
   cout << "   ELE_TYPE *obj;\n";
   cout << "   "; Emp_cont->cursxform(); CR;
   cout << "} CURS_TYPE;\n";
   cout << "\n" ;
   cout << "\n" ;
   cout << "typedef struct {\n" ;
   cout << "   "; Emp_cont->contxform(); CR;
   cout << "} CONT_TYPE;\n";
   cout << "\n" ;
   cout << "\n" ;
   cout << "EMP_TYPE rawdata[] = {\n" ;
   cout << "   { 10000,60,\"Biology\",\"Akers, Mark\" },\n" ;
   cout << "   { 10070,22,\"Biology\",\"Andrews, Kay\" },\n" ;
   cout << "   { 10020,21,\"Biology\",\"Alexander, Joe\" },\n" ;
   cout << "   { 10010,40,\"Biology\",\"Akin, Monica\" },\n" ;
   cout << "   { 10050,42,\"Biology\",\"Akerson, Suzanne\" },\n" ;
   cout << "   { 10040,53,\"Biology\",\"Akerson, Mary\" },\n" ;
   cout << "   { 10060,61,\"Biology\",\"Andrews, John\" },\n" ;
   cout << "   { 10030,23,\"Biology\",\"Akerson, Gwyn\" },\n" ;
   cout << "   { -1 }\n" ;
   cout << "};\n" ;
   cout << "\n" ;
   Emp_cont->contdecl(); CR;
   Oldcursor->cursdecl(); CR;
   Allcursor->cursdecl(); CR;
   cout << "\n" ;
   cout << "int main ( void ) \n" ;
   cout << "{\n" ;
   cout << "  "; Emp_cont->open_cont(); CR;
   cout << "  "; Oldcursor->init_curs(); CR;
   cout << "  "; Allcursor->init_curs(); CR;
   cout << "\n" ;
   cout << "  {\n" ;
   cout << "    EMP_TYPE *p;\n" ;
   cout << "    for (p = rawdata; p->empno != -1; p++) {\n" ;
   cout << "      "; Allcursor->insert( "*p" ); CR;
   cout << "    }\n" ;
   cout << "  }\n" ;
   cout << "\n" ;
   cout << "  printf(\"\\nall emp are:\\n\"); \n" ;
   cout << "  "; Allcursor->foreach(); Cr;
   cout << "  {\n" ;
   cout << "    EMP_TYPE r;\n" ;
   cout << "    "; Allcursor->getrec("r"); CR;
   cout << "    printf( \"%d %d %s\\n\", r.empno, r.age, r.name); \n" ;
   cout << "  }\n" ;
   cout << "\n" ;
   cout << "  printf( \"\\nold emp are:\\n\"); \n" ;
   cout << "  "; Oldcursor->foreach(); Cr;
   cout << "  {\n" ;
   cout << "    EMP_TYPE r;\n" ;
   cout << "    "; Oldcursor->getrec("r"); CR;
   cout << "    printf( \"%d %d %s\\n\", r.empno, r.age, r.name); \n" ;
   cout << "    "; Oldcursor->remove(); CR;
   cout << "  }\n" ;
   cout << "\n" ;
   cout << "  printf( \"\\nremaining emp are:\\n\"); \n" ;
   cout << "  "; Allcursor->foreach(); Cr;
   cout << "  {\n" ;
   cout << "    printf( \"%d %d %s\\n\", ";
   Allcursor->ref("empno"); cout << ",\n";
   cout << "             "; Allcursor->ref("age"); 
   cout << ", "; Allcursor->ref("name"); cout << ");\n" ;
   cout << "  }\n" ;
   cout << "\n" ;
   cout << "  "; Emp_cont->close_cont(); CR;
   cout << "  exit( 0 );\n" ;
   cout << "}\n" ;
}
